{% extends 'base.html' %}

{% block head %}
  <!-- script type="text/python">
    from frontend.tests import Test1
  </script -->
{% endblock %}


{% block content %}
  <h1>Run unittests</h1>
{% endblock %}
